<?php

/**
 * Overloaded Shopware class sBasket
 *
 * @category   Shopware
 * @package    Plugins
 * @subpackage Frontend
 * @author     LayoutBuddy Software GmbH
 * @copyright  Copyright © 2013 LayoutBuddy Software GmbH
 */

class Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_sBasket
{
    
     public function __construct()
     {

     }


    /**
     * Copied function from sBasket, small changes are marked with "LayoutBuddy"
     * Prevents resetting an article's price to it's original price (for LayoutBuddy products)
     */
    public function sUpdateArticle($instance, $id, $quantity, $lbCustomPrice = null, $lbLayoutId = null)
    {
        $quantity = intval($quantity);
        $id = intval($id);

        if (
            Shopware()->Events()->notifyUntil(
                'Shopware_Modules_Basket_UpdateArticle_Start',
                array('subject' => $instance, 'id' => $id, "quantity" => $quantity)
            )
        ) {
            return false;
        }

        if (!Shopware()->Session()->get('sessionId') || !$id) {
            return false;
        }

        list($queryAdditionalInfo, $quantity) = self::getAdditionalInfoForUpdateArticle($instance, $id, $quantity);
        $queryNewPrice = self::getPriceForUpdateArticle($instance, $id, $quantity, $queryAdditionalInfo);


        if (empty($queryNewPrice["price"]) && empty($queryNewPrice["config"])) {
            // If no price is set for default customer group, delete article from basket
            $instance->sDeleteArticle($id);
            return false;
        }


        // ----------------------------------- LayoutBuddy ----------------------------------- //
        if ($lbLayoutId && $lbCustomPrice) {
            $taxRate = $instance->sSYSTEM->sMODULES['sArticles']->getTaxRateByConditions($queryNewPrice["taxID"]);
            $netprice = $lbCustomPrice / (100 + $taxRate) * 100;
            $queryNewPrice["price"] = $netprice;
        }
        // ----------------------------------- LayoutBuddy ----------------------------------- //


        list($taxRate, $netPrice, $grossPrice) = self::getTaxesForUpdateArticle($instance, $quantity, $queryNewPrice, $queryAdditionalInfo);

        $sql = "
            UPDATE s_order_basket
            SET quantity = ?, price = ?, netprice = ?, currencyFactor = ?, tax_rate = ?
            WHERE id = ? AND sessionID = ? AND modus = 0
            ";
        $sql = Shopware()->Events()->filter(
            'Shopware_Modules_Basket_UpdateArticle_FilterSqlDefault',
            $sql,
            array(
                'subject' => $instance,
                'id' => $id,
                "quantity" => $quantity,
                "price" => $grossPrice,
                "netprice" => $netPrice,
                "currencyFactor" => $instance->sSYSTEM->sCurrency["factor"]
            )
        );

        if ($taxRate === false) {
            $taxRate = ($grossPrice == $netPrice) ? 0.00 : $queryNewPrice["tax"];
        }

        $update = Shopware()->Db()->query(
            $sql,
            array(
                $quantity,
                $grossPrice,
                $netPrice,
                $instance->sSYSTEM->sCurrency["factor"],
                $taxRate,
                $id,
                Shopware()->Session()->get('sessionId')
            )
        );

        $instance->sUpdateVoucher();

        if (!$update || !$queryNewPrice) {
            throw new Enlight_Exception("Basket Update ##01 Could not update quantity".$sql);
        }
    }

    

    /**
     * Copied function from sBasket, small changes are marked with "LayoutBuddy"
     * Prevents increasing the quantitiy of a single product in the cart, but add as a new one (for LayoutBuddy products)
     */
    public function sAddArticle($instance, $id, $quantity = 1, $isLayoutable = false)
    {
        
        // return $instance->sAddArticle($id, $quantity);

        $sessionId = Shopware()->Session()->get('sessionId');
        if (Shopware()->Session()->get('Bot') || empty($sessionId)) {
            return false;
        }

        $quantity = (empty($quantity) || !is_numeric($quantity)) ? 1 : (int) $quantity;
        if ($quantity <= 0) {
            $quantity = 1;
        }

        if (
            Shopware()->Events()->notifyUntil(
                'Shopware_Modules_Basket_AddArticle_Start',
                array(
                    'subject' => $instance,
                    'id' => $id,
                    "quantity" => $quantity
                )
            )
        ) {
            return false;
        }

        $article = self::getArticleForAddArticle($instance, $id);

        if (!$article) {
            return false;
        }

        $chkBasketForArticle = self::checkIfArticleIsInBasket(
            $instance,
            $article["articleID"],
            $article["ordernumber"],
            $sessionId
        );

        // Shopware 3.5.0 / sth / laststock - instock check
        if (!empty($chkBasketForArticle["id"])) {
            if (
                $article["laststock"] == true
                && $article["instock"] < ($chkBasketForArticle["quantity"] + $quantity)
            ) {
                $quantity -= $chkBasketForArticle["quantity"];
            }
        } else {
            if ($article["laststock"] == true && $article["instock"] <= $quantity) {
                $quantity = $article["instock"];
                if ($quantity <= 0) {
                    return;
                }
            }
        }

        // ----------------------------------- LayoutBuddy ----------------------------------- //
        if (!$isLayoutable && $chkBasketForArticle){
        // ----------------------------------- LayoutBuddy ----------------------------------- //
        //if ($chkBasketForArticle) {
            // Article is already in basket, update quantity
            $quantity += $chkBasketForArticle["quantity"];

            $instance->sUpdateArticle($chkBasketForArticle["id"], $quantity);
            return $chkBasketForArticle["id"];
        }

        $price = self::getPriceForAddArticle($instance, $article);

        // For variants, extend the article name
        if ($article["additionaltext"]) {
            $article["articleName"] .= " " . $article["additionaltext"];
        }

        if (!$article["shippingfree"]) {
            $article["shippingfree"] = "0";
        }

        // Check if article is an esd-article
        // - add flag to basket
        $getEsd = Shopware()->Db()->fetchOne(
            'SELECT s_articles_esd.id AS id, serials
            FROM s_articles_esd, s_articles_details
            WHERE s_articles_esd.articleID = ?
            AND s_articles_esd.articledetailsID = s_articles_details.id
            AND s_articles_details.ordernumber = ?',
            array($article["articleID"], $article["ordernumber"])
        );

        $sEsd = $getEsd ? '1' : '0';

        $quantity = (int) $quantity;
        $sql = "
            INSERT INTO s_order_basket (id, sessionID, userID, articlename, articleID,
                ordernumber, shippingfree, quantity, price, netprice,
                datum, esdarticle, partnerID, config)
            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )
        ";

        $params = array(
            '',
            (string) $sessionId,
            (string) Shopware()->Session()->get('sUserId'),
            $article["articleName"],
            $article["articleID"],
            (string) $article["ordernumber"],
            $article["shippingfree"],
            $quantity,
            $price["price"],
            $price["netprice"],
            date("Y-m-d H:i:s"),
            $sEsd,
            (string) Shopware()->Session()->get('sPartner'),
            ''
        );

        $sql = Shopware()->Events()->filter(
            'Shopware_Modules_Basket_AddArticle_FilterSql',
            $sql,
            array(
                'subject' => $instance,
                'article' => $article,
                'price' => $price,
                'esd' => $sEsd,
                'quantity' => $quantity,
                'partner' => Shopware()->Session()->get('sPartner')
            )
        );

        $result = Shopware()->Db()->query($sql, $params);

        if (!$result) {
            throw new Enlight_Exception("BASKET-INSERT #02 SQL-Error".$sql);
        }
        $insertId = Shopware()->Db()->lastInsertId();

        Shopware()->Db()->insert(
            's_order_basket_attributes',
            array(
                'basketID' => $insertId,
                'attribute1' => ''
            )
        );

        $instance->sUpdateArticle($insertId, $quantity);

        return $insertId;
    }










    private function getAdditionalInfoForUpdateArticle($instance, $id, $quantity)
    {
        // Query to get minimum surcharge
        $queryAdditionalInfo = Shopware()->Db()->fetchRow("
            SELECT s_articles_details.minpurchase, s_articles_details.purchasesteps,
            s_articles_details.maxpurchase, s_articles_details.purchaseunit,
            pricegroupID,pricegroupActive, s_order_basket.ordernumber, s_order_basket.articleID

            FROM s_articles, s_order_basket, s_articles_details
            WHERE s_order_basket.articleID = s_articles.id
            AND s_order_basket.ordernumber = s_articles_details.ordernumber
            AND s_order_basket.id = ?
            AND s_order_basket.sessionID = ?
            ",
            array($id, Shopware()->Session()->get('sessionId'))
        ) ? : array();

        // Check if quantity matches minimum purchase
        if (!$queryAdditionalInfo["minpurchase"]) {
            $queryAdditionalInfo["minpurchase"] = 1;
        }

        if ($quantity < $queryAdditionalInfo["minpurchase"]) {
            $quantity = $queryAdditionalInfo["minpurchase"];
        }

        // Check if quantity matches the step requirements
        if (!$queryAdditionalInfo["purchasesteps"]) {
            $queryAdditionalInfo["purchasesteps"] = 1;
        }

        if (($quantity / $queryAdditionalInfo["purchasesteps"]) != intval($quantity / $queryAdditionalInfo["purchasesteps"])) {
            $quantity = intval($quantity / $queryAdditionalInfo["purchasesteps"]) * $queryAdditionalInfo["purchasesteps"];
        }

        $maxPurchase = Shopware()->Config()->get('sMAXPURCHASE');
        if (empty($queryAdditionalInfo["maxpurchase"]) && !empty($maxPurchase)) {
            $queryAdditionalInfo["maxpurchase"] = $maxPurchase;
        }

        // Check if quantity matches max purchase
        if ($quantity > $queryAdditionalInfo["maxpurchase"] && !empty($queryAdditionalInfo["maxpurchase"])) {
            $quantity = $queryAdditionalInfo["maxpurchase"];
        }

        if (!empty($queryAdditionalInfo["purchaseunit"])) {
            $queryAdditionalInfo["purchaseunit"] = 1;
        }

        return array($queryAdditionalInfo, $quantity);
    }



    private function getPriceForUpdateArticle($instance, $id, $quantity, $queryAdditionalInfo)
    {
        // Price groups
        if ($queryAdditionalInfo["pricegroupActive"]) {
            $quantitySQL = 'AND s_articles_prices.from = 1 LIMIT 1';
        } else {
            $quantitySQL = Shopware()->Db()->quoteInto(
                ' AND s_articles_prices.from <= ? AND (s_articles_prices.to >= ? OR s_articles_prices.to = 0)',
                $quantity
            );
        }

        // Get the order number
        $sql = 'SELECT s_articles_prices.price AS price, taxID, s_core_tax.tax AS tax,
              tax_rate, s_articles_details.id AS articleDetailsID, s_articles_details.articleID,
              s_order_basket.config, s_order_basket.ordernumber
            FROM s_articles_details, s_articles_prices, s_order_basket,
              s_articles, s_core_tax
            WHERE s_order_basket.id = ? AND s_order_basket.sessionID = ?
            AND s_order_basket.ordernumber = s_articles_details.ordernumber
            AND s_articles_details.id=s_articles_prices.articledetailsID
            AND s_articles_details.articleID = s_articles.id
            AND s_articles.taxID = s_core_tax.id
            AND s_articles_prices.pricegroup = ?';

        $queryNewPrice = Shopware()->Db()->fetchRow(
            $sql . ' ' . $quantitySQL,
            array(
                $id,
                Shopware()->Session()->get('sessionId'),
                $instance->sSYSTEM->sUSERGROUP
            )
        ) ? : array();

        // Load prices from default group if article prices are not defined
        if (!$queryNewPrice["price"]) {
            // In the case no price is available for this customer group, use price of default customer group
            $sql = 'SELECT s_articles_prices.price AS price, taxID, s_core_tax.tax AS tax,
              s_articles_details.id AS articleDetailsID, s_articles_details.articleID,
              s_order_basket.config, s_order_basket.ordernumber
            FROM s_articles_details, s_articles_prices, s_order_basket,
              s_articles, s_core_tax
            WHERE s_order_basket.id = ? AND s_order_basket.sessionID = ?
            AND s_order_basket.ordernumber = s_articles_details.ordernumber
            AND s_articles_details.id=s_articles_prices.articledetailsID
            AND s_articles_details.articleID = s_articles.id
            AND s_articles.taxID = s_core_tax.id
            AND s_articles_prices.pricegroup = \'EK\'';

            $queryNewPrice = Shopware()->Db()->fetchRow(
                $sql . ' ' . $quantitySQL,
                array(
                    $id,
                    Shopware()->Session()->get('sessionId')
                )
            ) ? : array();
        }

        $queryNewPrice = Shopware()->Events()->filter('Shopware_Modules_Basket_getPriceForUpdateArticle_FilterPrice',
            $queryNewPrice,
            array(
                "id" => $id,
                'subject'=> $instance,
                "quantity" => $quantity
            )
        );

        return $queryNewPrice;
    }




    private function getTaxesForUpdateArticle($instance, $quantity, $queryNewPrice, $queryAdditionalInfo)
    {
        // Determinate tax rate for this cart position
        $taxRate = Shopware()->Modules()->Articles()->getTaxRateByConditions($queryNewPrice["taxID"]);

        $netPrice = $queryNewPrice["price"];

        // Recalculate price if purchase unit is set
        $grossPrice = Shopware()->Modules()->Articles()->sCalculatingPriceNum(
            $netPrice,
            $queryNewPrice["tax"],
            false,
            false,
            $queryNewPrice["taxID"],
            false,
            $queryNewPrice
        );

        // Check if tax free
        if (
            (Shopware()->Config()->get('sARTICLESOUTPUTNETTO') && !$instance->sSYSTEM->sUSERGROUPDATA["tax"])
            || (!$instance->sSYSTEM->sUSERGROUPDATA["tax"] && $instance->sSYSTEM->sUSERGROUPDATA["id"])
        ) {
            $netPrice = round($grossPrice, 2);
        } else {
            // Round to right value, if no purchase unit is set
            if ($queryAdditionalInfo["purchaseunit"] == 1) {
                $grossPrice = round($grossPrice, 2);
            }
            // Consider global discount for net price
            $netPrice = $grossPrice / (100 + $taxRate) * 100;
        }

        // Recalculate price per item, if purchase unit is set
        if ($queryAdditionalInfo["purchaseunit"] != 0) {
            $grossPrice = $grossPrice / $queryAdditionalInfo["purchaseunit"];
            $netPrice = $netPrice / $queryAdditionalInfo["purchaseunit"];
        }

        if (empty($instance->sSYSTEM->sCurrency["factor"])) {
            $instance->sSYSTEM->sCurrency["factor"] = 1;
        }

        if ($queryAdditionalInfo["pricegroupActive"]) {
            $grossPrice = Shopware()->Modules()->Articles()->sGetPricegroupDiscount(
                $instance->sSYSTEM->sUSERGROUP,
                $queryAdditionalInfo["pricegroupID"],
                $grossPrice,
                $quantity,
                false
            );
            $grossPrice = Shopware()->Modules()->Articles()->sRound($grossPrice);
            if (
                (Shopware()->Config()->get('sARTICLESOUTPUTNETTO') && !$instance->sSYSTEM->sUSERGROUPDATA["tax"]) ||
                (!$instance->sSYSTEM->sUSERGROUPDATA["tax"] && $instance->sSYSTEM->sUSERGROUPDATA["id"])
            ) {
                $netPrice = Shopware()->Modules()->Articles()->sRound(
                    Shopware()->Modules()->Articles()->sGetPricegroupDiscount(
                        $instance->sSYSTEM->sUSERGROUP,
                        $queryAdditionalInfo["pricegroupID"],
                        $netPrice,
                        $quantity,
                        false
                    )
                );
            } else {
                $netPrice = $grossPrice / (100 + $taxRate) * 100;
                $netPrice = number_format($netPrice, 3, ".", "");
            }
        }
        return array($taxRate, $netPrice, $grossPrice);
    }


    /**
     * Get article data for sAddArticle
     *
     * @param int $id Article ordernumber
     * @return array|false Article data, or false if none found
     */
    private function getArticleForAddArticle($instance, $id)
    {
        $sql = "
            SELECT s_articles.id AS articleID, s_articles.main_detail_id, name AS articleName, taxID,
              additionaltext, s_articles_details.shippingfree, s_articles.laststock, instock,
              s_articles_details.id as articledetailsID, ordernumber,
              s_articles.configurator_set_id
            FROM s_articles, s_articles_details
            WHERE s_articles_details.ordernumber = ?
            AND s_articles_details.articleID = s_articles.id
            AND s_articles.active = 1
            AND (
                SELECT articleID
                FROM s_articles_avoid_customergroups
                WHERE articleID = s_articles.id AND customergroupID = ?
            ) IS NULL
        ";

        $article = Shopware()->Db()->fetchRow(
            $sql,
            array($id, $instance->sSYSTEM->sUSERGROUPDATA["id"])
        );

        $article = Shopware()->Events()->filter('Shopware_Modules_Basket_getArticleForAddArticle_FilterArticle',
            $article,
            array(
                "id" => $id,
                'subject'=> $instance,
                "partner" => $instance->sSYSTEM->_SESSION["sPartner"]
            )
        );

        if (!$article) {
            return false;
        }

        $article = Shopware()->Modules()->Articles()->sGetTranslation(
            $article,
            $article['articleID'],
            "article"
        );

        $article = Shopware()->Modules()->Articles()->sGetTranslation(
            $article,
            $article['articledetailsID'],
            "variant"
        );

        if ($article['configurator_set_id'] > 0) {
            $product = new StoreFrontBundle\Struct\ListProduct(
                (int) $article['articleID'],
                (int) $article["articledetailsID"],
                $article['ordernumber']
            );
            $product->setAdditional($article['additionaltext']);

            $context = Shopware()->Container()->get('shopware_storefront.context_service')->getShopContext();
            $product = Shopware()->Container()->get('shopware_storefront.additional_text_service')->buildAdditionalText($product, $context);
            $article['additionaltext'] = $product->getAdditional();
        }

        return $article;
    }



    /**
     * Check if article is already in basket
     *
     * @param int    $articleId
     * @param string $ordernumber
     * @param string $sessionId
     * @return array Example: ["id" => "731", "quantity" => "100"]
     */
    private function checkIfArticleIsInBasket($instance, $articleId, $ordernumber, $sessionId)
    {
        $builder = Shopware()->Models()->getConnection()->createQueryBuilder();

        $builder->select('id', 'quantity')
            ->from('s_order_basket', 'basket')
            ->where('articleID = :articleId')
            ->andWhere('sessionID = :sessionId')
            ->andWhere('ordernumber = :ordernumber')
            ->andWhere('modus != 1')
            ->setParameter('articleId', $articleId)
            ->setParameter('sessionId', $sessionId)
            ->setParameter('ordernumber', $ordernumber);

        Shopware()->Events()->notify(
            'Shopware_Modules_Basket_AddArticle_CheckBasketForArticle',
            array(
                'queryBuilder' => $builder,
                'subject'      => $instance
            )
        );

        /**@var $statement \Doctrine\DBAL\Driver\ResultStatement */
        $statement = $builder->execute();

        return $statement->fetch() ?: array();
    }




    /**
     * @param $article
     * @return array
     * @throws Enlight_Exception
     */
    private function getPriceForAddArticle($instance, $article)
    {
        // Read price from default price table
        $price = Shopware()->Db()->fetchRow(
            'SELECT price, s_core_tax.tax AS tax
            FROM s_articles_prices, s_core_tax
            WHERE s_articles_prices.pricegroup = ?
            AND s_articles_prices.articledetailsID = ?
            AND s_core_tax.id = ?',
            array(
                $instance->sSYSTEM->sUSERGROUP,
                $article["articledetailsID"],
                $article["taxID"]
            )
        ) ? : array();

        if (empty($price["price"])) {
            $price = Shopware()->Db()->fetchRow(
                'SELECT price, s_core_tax.tax AS tax
                FROM s_articles_prices, s_core_tax
                WHERE s_articles_prices.pricegroup = \'EK\'
                AND s_articles_prices.articledetailsID = ?
                AND s_core_tax.id = ?',
                array($article["articledetailsID"], $article["taxID"])
            ) ? : array();
        }

        if (!$price["price"] && !$article["free"]) {
            // No price could acquired
            throw new Enlight_Exception("BASKET-INSERT #01 No price acquired");
        }

        // If configuration article
        if (
            (Shopware()->Config()->get('sARTICLESOUTPUTNETTO') && !$instance->sSYSTEM->sUSERGROUPDATA["tax"])
            || (!$instance->sSYSTEM->sUSERGROUPDATA["tax"] && $instance->sSYSTEM->sUSERGROUPDATA["id"])
        ) {
            // If netto set both values to net-price
            $price["price"] = Shopware()->Modules()->Articles()->sCalculatingPriceNum(
                $price["price"],
                $price["tax"],
                false,
                false,
                $article["taxID"],
                false,
                $article
            );
            $price["netprice"] = $price["price"];
        } else {
            // If brutto, save net
            $price["netprice"] = $price["price"];
            $price["price"] = Shopware()->Modules()->Articles()->sCalculatingPriceNum(
                $price["price"],
                $price["tax"],
                false,
                false,
                $article["taxID"],
                false,
                $article
            );
        }
        return $price;
    }
    
}
