<?php

/**
 * Backend controller to show LayoutBuddy tab
 *
 * @category   Shopware
 * @package    Plugins
 * @subpackage Frontend
 * @author     LayoutBuddy Software GmbH
 * @copyright  Copyright © 2013 LayoutBuddy Software GmbH
 */

class Shopware_Controllers_Backend_LyByLayoutBuddyOrdersTab extends Enlight_Controller_Action
{
    
    public static $repository = null;

    /**
     * Helper function to get instance of repository
     */
    protected function getRepository()
    {
        if(self::$repository === null) {
            self::$repository = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
        }
        return self::$repository;
    }

    /**
     * Resets renderer for all actions
     */
    public function preDispatch()
    {
        Shopware()->Plugins()->Controller()->ViewRenderer()->setNoRender();
    }

    /**
     * Returns list of widgets to download final PDFs
     */
    public function widgetsAction()
    {

        $config = Shopware()->Plugins()->Frontend()->LyByLayoutBuddy()->Config();
        if ((empty($config->lbPrivateKey) || !$config->lbPrivateKey) ||
            (empty($config->lbPublicKey) || !$config->lbPublicKey)) {
            $this->Response()->setBody('LayoutBuddy not configured.');
            return;
        }

        if (!$id = intval($this->Request()->getParam('id'))) return;

        $builder = $this->getRepository()->getOrdersQueryBuilder(array(array('property' => 'orders.id', 'value' => $id)));
        $order = $builder->getQuery()->getOneOrNullResult(2);

        if (!$order) return;

        $api = new Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_Api($config->lbPublicKey, $config->lbPrivateKey);
        $html = '';

        foreach ($order['details'] as $pos => $detail) {

            if (isset($detail['attribute']['lbLayoutId']) && ($layoutId = $detail['attribute']['lbLayoutId'])) {
                
                $params = array(       
                    'id' => $layoutId,
                    'order_code' => $order['number'],
                    'item_code' => $detail['id'],
                    'item_number' => ($pos + 1),
                    'item_qty' => $detail['quantity'],
                    'show_options' => true,
                );

                $widgetUrl = $api->buildSignedRequestUrl('widget', $params);

                $html .= '<h1 style="margin-top: 10px">Pos. ' . ($pos + 1) . ': ' . $detail['quantity'] . 'x ' . $detail['articleName'] . ' (' . $detail['articleNumber'] . ')</h1>'
                      .  '<p  style="margin-bottom: 10px">Layout ID: ' . $layoutId . '</p>'
                      .  '<div style="width: 100%; background: #f5f5f5; padding: 5px; margin-bottom: 20px">'
                      .  '<iframe src="' . $widgetUrl .'" scrolling="no" frameborder="0" style="border: none; width: 100%; height: 120px; min-height: 0"></iframe>'
                      .  '</div>'
                ;

            }
        }

        $this->Response()->setBody($html);

    }

    /**
     * Returns list of configurations for products
     */
    public function configsAction()
    {
        $config = Shopware()->Plugins()->Frontend()->LyByLayoutBuddy()->Config();
        if ((empty($config->lbPrivateKey) || !$config->lbPrivateKey) ||
            (empty($config->lbPublicKey) || !$config->lbPublicKey)) {
            $this->Response()->setBody(json_encode(array()));
            return;
        }


        $result = array();
        try {
            $api = new Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_Api($config->lbPublicKey, $config->lbPrivateKey);
            $configs = $api->jsonRequest('listConfigs', array(), true);
            foreach ($configs as $config) {
                $result[] = array(
                    'id' => $config->id,
                    'label' => $config->id . ': ' . $config->name . ' (' . $config->base . ')',
                );
            }
        } catch (Exception $e) {

        }

        $this->Response()->setBody(json_encode($result));
    }
}